function [err, timepoints, species_out, observables_out] = LigandModel5_refined( timepoints, species_init, parameters, suppress_plot )
%LIGANDMODEL5_REFINED Integrate reaction network and plot observables.
%   Integrates the reaction network corresponding to the BioNetGen model
%   'LigandModel5_refined' and then (optionally) plots the observable trajectories,
%   or species trajectories if no observables are defined. Trajectories are
%   generated using either default or user-defined parameters and initial
%   species values. Integration is performed by the MATLAB stiff solver
%   'ode15s'. LIGANDMODEL5_REFINED returns an error value, a vector of timepoints,
%   species trajectories, and observable trajectories.
%   
%   [err, timepoints, species_out, observables_out]
%        = LigandModel5_refined( timepoints, species_init, parameters, suppress_plot )
%
%   INPUTS:
%   -------
%   species_init    : row vector of 74 initial species populations.
%   timepoints      : column vector of time points returned by integrator.
%   parameters      : row vector of 6 model parameters.
%   suppress_plot   : 0 if a plot is desired (default), 1 if plot is suppressed.
%
%   Note: to specify default value for an input argument, pass the empty array.
%
%   OUTPUTS:
%   --------
%   err             : 0 if the integrator exits without error, non-zero otherwise.
%   timepoints      : a row vector of timepoints returned by the integrator.
%   species_out     : array of species population trajectories
%                        (columns correspond to species, rows correspond to time).
%   observables_out : array of observable trajectories
%                        (columns correspond to observables, rows correspond to time).
%
%   QUESTIONS about the BNG Mfile generator?  Email justinshogg@gmail.com



%% Process input arguments

% define any missing arguments
if ( nargin < 1 )
    timepoints = [];
end

if ( nargin < 2 )
    species_init = [];
end

if ( nargin < 3 )
    parameters = [];
end

if ( nargin < 4 )
    suppress_plot = 0;
end


% initialize outputs (to avoid error msgs if script terminates early
err = 0;
species_out     = [];
observables_out = [];


% setup default parameters, if necessary
if ( isempty(parameters) )
   parameters = [ 1, 1, 1, 1, 1, 1 ];
end
% check that parameters has proper dimensions
if (  size(parameters,1) ~= 1  ||  size(parameters,2) ~= 6  )
    fprintf( 1, 'Error: size of parameter argument is invalid! Correct size = [1 6].\n' );
    err = 1;
    return;
end

% setup default initial values, if necessary
if ( isempty(species_init) )
   species_init = initialize_species( parameters );
end
% check that species_init has proper dimensions
if (  size(species_init,1) ~= 1  ||  size(species_init,2) ~= 74  )
    fprintf( 1, 'Error: size of species_init argument is invalid! Correct size = [1 74].\n' );
    err = 1;
    return;
end

% setup default timepoints, if necessary
if ( isempty(timepoints) )
   timepoints = linspace(0,10,20+1)';
end
% check that timepoints has proper dimensions
if (  size(timepoints,1) < 2  ||  size(timepoints,2) ~= 1  )
    fprintf( 1, 'Error: size of timepoints argument is invalid! Correct size = [t 1], t>1.\n' );
    err = 1;
    return;
end

% setup default suppress_plot, if necessary
if ( isempty(suppress_plot) )
   suppress_plot = 0;
end
% check that suppress_plot has proper dimensions
if ( size(suppress_plot,1) ~= 1  ||  size(suppress_plot,2) ~= 1 )
    fprintf( 1, 'Error: suppress_plots argument should be a scalar!\n' );
    err = 1;
    return;
end

% define parameter labels (this is for the user's reference!)
param_labels = { 'k1', 'k2', 'k3', 'k4', 'p1', 'p2' };



%% Integrate Network Model
 
% calculate expressions
[expressions] = calc_expressions( parameters );

% set ODE integrator options
opts = odeset( 'RelTol',   1e-8,   ...
               'AbsTol',   0.0001,   ...
               'Stats',    'off',  ...
               'BDF',      'off',    ...
               'MaxOrder', 5   );


% define derivative function
rhs_fcn = @(t,y)( calc_species_deriv( t, y, expressions ) );

% simulate model system (stiff integrator)
try 
    [~, species_out] = ode15s( rhs_fcn, timepoints, species_init', opts );
    if(length(timepoints) ~= size(species_out,1))
        exception = MException('ODE15sError:MissingOutput','Not all timepoints output\n');
        throw(exception);
    end
catch
    err = 1;
    fprintf( 1, 'Error: some problem encountered while integrating ODE network!\n' );
    return;
end

% calculate observables
observables_out = zeros( length(timepoints), 6 );
for t = 1 : length(timepoints)
    observables_out(t,:) = calc_observables( species_out(t,:), expressions );
end


%% Plot Output, if desired

if ( ~suppress_plot )
    
    % define plot labels
    observable_labels = { 'RM', 'RD_b1', 'RD_b2', 'RD', 'P_RM', 'P_RD' };

    % construct figure
    plot(timepoints,observables_out);
    title('LigandModel5_refined observables','fontSize',14,'Interpreter','none');
    axis([0 timepoints(end) 0 inf]);
    legend(observable_labels,'fontSize',10,'Interpreter','none');
    xlabel('time','fontSize',12,'Interpreter','none');
    ylabel('number or concentration','fontSize',12,'Interpreter','none');

end


%~~~~~~~~~~~~~~~~~~~~~%
% END of main script! %
%~~~~~~~~~~~~~~~~~~~~~%


% initialize species function
function [species_init] = initialize_species( params )

    species_init = zeros(1,74);
    species_init(1) = 1;
    species_init(2) = 1;
    species_init(3) = 1;
    species_init(4) = 1;
    species_init(5) = 0;
    species_init(6) = 0;
    species_init(7) = 0;
    species_init(8) = 0;
    species_init(9) = 0;
    species_init(10) = 0;
    species_init(11) = 0;
    species_init(12) = 0;
    species_init(13) = 0;
    species_init(14) = 0;
    species_init(15) = 0;
    species_init(16) = 0;
    species_init(17) = 0;
    species_init(18) = 0;
    species_init(19) = 0;
    species_init(20) = 0;
    species_init(21) = 0;
    species_init(22) = 0;
    species_init(23) = 0;
    species_init(24) = 0;
    species_init(25) = 0;
    species_init(26) = 0;
    species_init(27) = 0;
    species_init(28) = 0;
    species_init(29) = 0;
    species_init(30) = 0;
    species_init(31) = 0;
    species_init(32) = 0;
    species_init(33) = 0;
    species_init(34) = 0;
    species_init(35) = 0;
    species_init(36) = 0;
    species_init(37) = 0;
    species_init(38) = 0;
    species_init(39) = 0;
    species_init(40) = 0;
    species_init(41) = 0;
    species_init(42) = 0;
    species_init(43) = 0;
    species_init(44) = 0;
    species_init(45) = 0;
    species_init(46) = 0;
    species_init(47) = 0;
    species_init(48) = 0;
    species_init(49) = 0;
    species_init(50) = 0;
    species_init(51) = 0;
    species_init(52) = 0;
    species_init(53) = 0;
    species_init(54) = 0;
    species_init(55) = 0;
    species_init(56) = 0;
    species_init(57) = 0;
    species_init(58) = 0;
    species_init(59) = 0;
    species_init(60) = 0;
    species_init(61) = 0;
    species_init(62) = 0;
    species_init(63) = 0;
    species_init(64) = 0;
    species_init(65) = 0;
    species_init(66) = 0;
    species_init(67) = 0;
    species_init(68) = 0;
    species_init(69) = 0;
    species_init(70) = 0;
    species_init(71) = 0;
    species_init(72) = 0;
    species_init(73) = 0;
    species_init(74) = 0;

end


% user-defined functions



% Calculate expressions
function [ expressions ] = calc_expressions ( parameters )

    expressions = zeros(1,6);
    expressions(1) = parameters(1);
    expressions(2) = parameters(2);
    expressions(3) = parameters(3);
    expressions(4) = parameters(4);
    expressions(5) = parameters(5);
    expressions(6) = parameters(6);
   
end



% Calculate observables
function [ observables ] = calc_observables ( species, expressions )

    observables = zeros(1,6);
    observables(1) = species(5) +species(6) +species(7) +2*species(14) +species(15) +species(16) +species(17) +species(18) +species(26) +species(27) +species(28) +species(36) +species(37) +2*species(38) +species(39) +species(40) +species(41) +species(42) +2*species(43) +species(44) +species(45) +species(54) +species(55) +species(56) +species(57) +species(58) +species(59) +species(60) +species(61) +2*species(62) +species(63) +species(64) +species(65) +species(66) +species(67) +species(68) +species(69) +species(70) +species(71) +species(72) +species(73) +species(74);
    observables(2) = species(8) +species(10) +species(11) +species(15) +species(17) +species(29) +species(30) +species(31) +species(39) +species(44) +species(54) +species(56) +species(63) +species(71);
    observables(3) = species(9) +species(12) +species(13) +species(16) +species(18) +species(32) +species(33) +species(34) +species(40) +species(45) +species(55) +species(57) +species(64) +species(72);
    observables(4) = species(19) +species(20) +species(21) +species(22) +species(23) +species(24) +species(25) +species(36) +species(37) +species(41) +species(42) +species(46) +species(47) +species(48) +species(50) +species(51) +species(52) +species(53) +species(58) +species(59) +species(60) +species(61) +species(65) +species(66) +species(67) +species(68) +species(69) +species(70) +species(73) +species(74);
    observables(5) = species(26) +species(27) +species(28) +species(35) +species(38) +species(43) +species(54) +species(55) +species(56) +species(57) +species(60) +species(61) +2*species(62) +species(63) +species(64) +species(65) +species(66) +species(69) +species(70) +species(71) +species(72) +species(73) +species(74);
    observables(6) = species(29) +species(30) +species(31) +species(32) +species(33) +species(34) +species(39) +species(40) +species(44) +species(45) +species(46) +species(47) +species(48) +species(49) +species(50) +species(51) +species(52) +species(53) +species(58) +species(59) +species(63) +species(64) +species(65) +species(66) +species(67) +species(68) +species(71) +species(72) +species(73) +species(74);

end


% Calculate ratelaws
function [ ratelaws ] = calc_ratelaws ( species, expressions, observables )

    ratelaws = zeros(1,6);
    ratelaws(1) = expressions(1)*species(1)*species(3);
    ratelaws(2) = expressions(1)*species(1)*species(4);
    ratelaws(3) = expressions(1)*species(1)*species(4);
    ratelaws(4) = expressions(1)*species(2)*species(3);
    ratelaws(5) = expressions(1)*species(2)*species(3);
    ratelaws(6) = expressions(1)*species(2)*species(4);
    ratelaws(7) = expressions(1)*species(2)*species(4);
    ratelaws(8) = expressions(1)*species(2)*species(4);
    ratelaws(9) = expressions(1)*species(2)*species(4);
    ratelaws(10) = expressions(2)*species(5);
    ratelaws(11) = expressions(1)*species(1)*species(7);
    ratelaws(12) = expressions(1)*species(1)*species(11);
    ratelaws(13) = expressions(1)*species(1)*species(13);
    ratelaws(14) = expressions(2)*species(6);
    ratelaws(15) = expressions(1)*species(1)*species(6);
    ratelaws(16) = expressions(1)*species(1)*species(10);
    ratelaws(17) = expressions(1)*species(1)*species(12);
    ratelaws(18) = expressions(2)*species(7);
    ratelaws(19) = expressions(1)*species(9)*species(3);
    ratelaws(20) = expressions(1)*species(12)*species(3);
    ratelaws(21) = expressions(1)*species(13)*species(3);
    ratelaws(22) = expressions(2)*species(8);
    ratelaws(23) = expressions(1)*species(8)*species(3);
    ratelaws(24) = expressions(1)*species(10)*species(3);
    ratelaws(25) = expressions(1)*species(11)*species(3);
    ratelaws(26) = expressions(2)*species(9);
    ratelaws(27) = expressions(2)*species(10);
    ratelaws(28) = expressions(3)*species(10);
    ratelaws(29) = expressions(2)*species(11);
    ratelaws(30) = expressions(3)*species(11);
    ratelaws(31) = expressions(2)*species(12);
    ratelaws(32) = expressions(3)*species(12);
    ratelaws(33) = expressions(2)*species(13);
    ratelaws(34) = expressions(3)*species(13);
    ratelaws(35) = expressions(5)*species(5);
    ratelaws(36) = expressions(5)*species(6);
    ratelaws(37) = expressions(5)*species(7);
    ratelaws(38) = expressions(5)*species(8);
    ratelaws(39) = expressions(5)*species(10);
    ratelaws(40) = expressions(5)*species(11);
    ratelaws(41) = expressions(5)*species(9);
    ratelaws(42) = expressions(5)*species(12);
    ratelaws(43) = expressions(5)*species(13);
    ratelaws(44) = expressions(2)*species(26);
    ratelaws(45) = expressions(1)*species(1)*species(21);
    ratelaws(46) = expressions(1)*species(1)*species(23);
    ratelaws(47) = expressions(1)*species(1)*species(28);
    ratelaws(48) = expressions(1)*species(1)*species(31);
    ratelaws(49) = expressions(1)*species(1)*species(34);
    ratelaws(50) = expressions(2)*species(14);
    ratelaws(51) = expressions(2)*species(15);
    ratelaws(52) = expressions(2)*species(16);
    ratelaws(53) = expressions(2)*species(27);
    ratelaws(54) = expressions(1)*species(1)*species(20);
    ratelaws(55) = expressions(1)*species(1)*species(22);
    ratelaws(56) = expressions(1)*species(1)*species(27);
    ratelaws(57) = expressions(1)*species(1)*species(30);
    ratelaws(58) = expressions(1)*species(1)*species(33);
    ratelaws(59) = expressions(2)*species(14);
    ratelaws(60) = expressions(2)*species(17);
    ratelaws(61) = expressions(2)*species(18);
    ratelaws(62) = expressions(2)*species(28);
    ratelaws(63) = expressions(1)*species(16)*species(3);
    ratelaws(64) = expressions(1)*species(18)*species(3);
    ratelaws(65) = expressions(1)*species(32)*species(3);
    ratelaws(66) = expressions(1)*species(33)*species(3);
    ratelaws(67) = expressions(1)*species(34)*species(3);
    ratelaws(68) = expressions(2)*species(19);
    ratelaws(69) = expressions(2)*species(20);
    ratelaws(70) = expressions(2)*species(21);
    ratelaws(71) = expressions(2)*species(29);
    ratelaws(72) = expressions(1)*species(15)*species(3);
    ratelaws(73) = expressions(1)*species(17)*species(3);
    ratelaws(74) = expressions(1)*species(29)*species(3);
    ratelaws(75) = expressions(1)*species(30)*species(3);
    ratelaws(76) = expressions(1)*species(31)*species(3);
    ratelaws(77) = expressions(2)*species(19);
    ratelaws(78) = expressions(2)*species(22);
    ratelaws(79) = expressions(2)*species(23);
    ratelaws(80) = expressions(2)*species(32);
    ratelaws(81) = expressions(2)*species(30);
    ratelaws(82) = expressions(3)*species(30);
    ratelaws(83) = expressions(4)*species(24);
    ratelaws(84) = expressions(2)*species(31);
    ratelaws(85) = expressions(3)*species(31);
    ratelaws(86) = expressions(4)*species(25);
    ratelaws(87) = expressions(2)*species(33);
    ratelaws(88) = expressions(3)*species(33);
    ratelaws(89) = expressions(4)*species(25);
    ratelaws(90) = expressions(2)*species(34);
    ratelaws(91) = expressions(3)*species(34);
    ratelaws(92) = expressions(4)*species(24);
    ratelaws(93) = expressions(5)*species(14);
    ratelaws(94) = expressions(5)*species(14);
    ratelaws(95) = expressions(5)*species(15);
    ratelaws(96) = expressions(5)*species(16);
    ratelaws(97) = expressions(5)*species(17);
    ratelaws(98) = expressions(5)*species(18);
    ratelaws(99) = expressions(5)*species(15);
    ratelaws(100) = expressions(5)*species(17);
    ratelaws(101) = 2*expressions(5)*species(19);
    ratelaws(102) = 2*expressions(5)*species(20);
    ratelaws(103) = 2*expressions(5)*species(21);
    ratelaws(104) = 2*expressions(5)*species(22);
    ratelaws(105) = 2*expressions(5)*species(23);
    ratelaws(106) = 2*expressions(5)*species(24);
    ratelaws(107) = 2*expressions(5)*species(25);
    ratelaws(108) = expressions(5)*species(16);
    ratelaws(109) = expressions(5)*species(18);
    ratelaws(110) = expressions(1)*species(35)*species(3);
    ratelaws(111) = expressions(1)*species(1)*species(48);
    ratelaws(112) = expressions(1)*species(1)*species(51);
    ratelaws(113) = expressions(1)*species(35)*species(4);
    ratelaws(114) = expressions(1)*species(35)*species(7);
    ratelaws(115) = expressions(1)*species(35)*species(11);
    ratelaws(116) = expressions(1)*species(35)*species(13);
    ratelaws(117) = expressions(1)*species(35)*species(21);
    ratelaws(118) = expressions(1)*species(35)*species(23);
    ratelaws(119) = expressions(1)*species(35)*species(28);
    ratelaws(120) = expressions(1)*species(35)*species(31);
    ratelaws(121) = expressions(1)*species(35)*species(34);
    ratelaws(122) = expressions(1)*species(35)*species(48);
    ratelaws(123) = expressions(1)*species(35)*species(51);
    ratelaws(124) = expressions(2)*species(36);
    ratelaws(125) = expressions(2)*species(37);
    ratelaws(126) = expressions(2)*species(38);
    ratelaws(127) = expressions(2)*species(39);
    ratelaws(128) = expressions(2)*species(40);
    ratelaws(129) = expressions(2)*species(43);
    ratelaws(130) = expressions(2)*species(54);
    ratelaws(131) = expressions(2)*species(55);
    ratelaws(132) = expressions(1)*species(1)*species(47);
    ratelaws(133) = expressions(1)*species(1)*species(50);
    ratelaws(134) = expressions(1)*species(35)*species(4);
    ratelaws(135) = expressions(1)*species(35)*species(6);
    ratelaws(136) = expressions(1)*species(35)*species(10);
    ratelaws(137) = expressions(1)*species(35)*species(12);
    ratelaws(138) = expressions(1)*species(35)*species(20);
    ratelaws(139) = expressions(1)*species(35)*species(22);
    ratelaws(140) = expressions(1)*species(35)*species(27);
    ratelaws(141) = expressions(1)*species(35)*species(30);
    ratelaws(142) = expressions(1)*species(35)*species(33);
    ratelaws(143) = expressions(1)*species(35)*species(47);
    ratelaws(144) = expressions(1)*species(35)*species(50);
    ratelaws(145) = expressions(2)*species(38);
    ratelaws(146) = expressions(2)*species(41);
    ratelaws(147) = expressions(2)*species(42);
    ratelaws(148) = expressions(2)*species(43);
    ratelaws(149) = expressions(2)*species(44);
    ratelaws(150) = expressions(2)*species(45);
    ratelaws(151) = expressions(2)*species(56);
    ratelaws(152) = expressions(2)*species(57);
    ratelaws(153) = expressions(1)*species(40)*species(3);
    ratelaws(154) = expressions(1)*species(45)*species(3);
    ratelaws(155) = expressions(1)*species(49)*species(3);
    ratelaws(156) = expressions(1)*species(55)*species(3);
    ratelaws(157) = expressions(1)*species(57)*species(3);
    ratelaws(158) = expressions(2)*species(36);
    ratelaws(159) = expressions(2)*species(41);
    ratelaws(160) = expressions(2)*species(46);
    ratelaws(161) = expressions(2)*species(47);
    ratelaws(162) = expressions(2)*species(48);
    ratelaws(163) = expressions(1)*species(39)*species(3);
    ratelaws(164) = expressions(1)*species(44)*species(3);
    ratelaws(165) = expressions(1)*species(49)*species(3);
    ratelaws(166) = expressions(1)*species(54)*species(3);
    ratelaws(167) = expressions(1)*species(56)*species(3);
    ratelaws(168) = expressions(2)*species(37);
    ratelaws(169) = expressions(2)*species(42);
    ratelaws(170) = expressions(2)*species(46);
    ratelaws(171) = expressions(2)*species(50);
    ratelaws(172) = expressions(2)*species(51);
    ratelaws(173) = expressions(1)*species(49)*species(4);
    ratelaws(174) = expressions(4)*species(52);
    ratelaws(175) = expressions(1)*species(49)*species(4);
    ratelaws(176) = expressions(4)*species(53);
    ratelaws(177) = expressions(1)*species(49)*species(4);
    ratelaws(178) = expressions(4)*species(53);
    ratelaws(179) = expressions(1)*species(49)*species(4);
    ratelaws(180) = expressions(4)*species(52);
    ratelaws(181) = expressions(5)*species(36);
    ratelaws(182) = expressions(5)*species(37);
    ratelaws(183) = expressions(5)*species(38);
    ratelaws(184) = expressions(5)*species(39);
    ratelaws(185) = expressions(5)*species(40);
    ratelaws(186) = expressions(5)*species(41);
    ratelaws(187) = expressions(5)*species(42);
    ratelaws(188) = expressions(5)*species(43);
    ratelaws(189) = expressions(5)*species(44);
    ratelaws(190) = expressions(5)*species(45);
    ratelaws(191) = expressions(6)*species(35);
    ratelaws(192) = 2*expressions(5)*species(36);
    ratelaws(193) = 2*expressions(5)*species(37);
    ratelaws(194) = 2*expressions(5)*species(41);
    ratelaws(195) = 2*expressions(5)*species(42);
    ratelaws(196) = expressions(5)*species(54);
    ratelaws(197) = expressions(5)*species(56);
    ratelaws(198) = expressions(5)*species(55);
    ratelaws(199) = expressions(5)*species(57);
    ratelaws(200) = expressions(6)*species(49);
    ratelaws(201) = expressions(2)*species(58);
    ratelaws(202) = expressions(2)*species(59);
    ratelaws(203) = expressions(2)*species(60);
    ratelaws(204) = expressions(2)*species(61);
    ratelaws(205) = expressions(2)*species(62);
    ratelaws(206) = expressions(2)*species(63);
    ratelaws(207) = expressions(2)*species(64);
    ratelaws(208) = expressions(2)*species(65);
    ratelaws(209) = expressions(2)*species(66);
    ratelaws(210) = expressions(2)*species(62);
    ratelaws(211) = expressions(2)*species(67);
    ratelaws(212) = expressions(2)*species(68);
    ratelaws(213) = expressions(2)*species(69);
    ratelaws(214) = expressions(2)*species(70);
    ratelaws(215) = expressions(2)*species(71);
    ratelaws(216) = expressions(2)*species(72);
    ratelaws(217) = expressions(2)*species(73);
    ratelaws(218) = expressions(2)*species(74);
    ratelaws(219) = expressions(1)*species(64)*species(3);
    ratelaws(220) = expressions(1)*species(72)*species(3);
    ratelaws(221) = expressions(2)*species(58);
    ratelaws(222) = expressions(2)*species(60);
    ratelaws(223) = expressions(2)*species(65);
    ratelaws(224) = expressions(2)*species(67);
    ratelaws(225) = expressions(2)*species(69);
    ratelaws(226) = expressions(2)*species(73);
    ratelaws(227) = expressions(1)*species(63)*species(3);
    ratelaws(228) = expressions(1)*species(71)*species(3);
    ratelaws(229) = expressions(2)*species(59);
    ratelaws(230) = expressions(2)*species(61);
    ratelaws(231) = expressions(2)*species(66);
    ratelaws(232) = expressions(2)*species(68);
    ratelaws(233) = expressions(2)*species(70);
    ratelaws(234) = expressions(2)*species(74);
    ratelaws(235) = expressions(5)*species(58);
    ratelaws(236) = expressions(5)*species(59);
    ratelaws(237) = expressions(5)*species(67);
    ratelaws(238) = expressions(5)*species(68);
    ratelaws(239) = 2*expressions(5)*species(60);
    ratelaws(240) = 2*expressions(5)*species(61);
    ratelaws(241) = 2*expressions(5)*species(69);
    ratelaws(242) = 2*expressions(5)*species(70);

end

% Calculate species derivatives
function [ Dspecies ] = calc_species_deriv ( time, species, expressions )
    
    % initialize derivative vector
    Dspecies = zeros(74,1);
    
    % update observables
    [ observables ] = calc_observables( species, expressions );
    
    % update ratelaws
    [ ratelaws ] = calc_ratelaws( species, expressions, observables );
                        
    % calculate derivatives
    Dspecies(1) = -ratelaws(1) -ratelaws(2) -ratelaws(3) +ratelaws(10) -ratelaws(11) -ratelaws(12) -ratelaws(13) +ratelaws(14) -ratelaws(15) -ratelaws(16) -ratelaws(17) +ratelaws(18) -ratelaws(45) -ratelaws(46) -ratelaws(47) -ratelaws(48) -ratelaws(49) +ratelaws(50) +ratelaws(51) +ratelaws(52) -ratelaws(54) -ratelaws(55) -ratelaws(56) -ratelaws(57) -ratelaws(58) +ratelaws(59) +ratelaws(60) +ratelaws(61) -ratelaws(111) -ratelaws(112) +ratelaws(124) +ratelaws(125) +ratelaws(126) +ratelaws(127) +ratelaws(128) -ratelaws(132) -ratelaws(133) +ratelaws(146) +ratelaws(147) +ratelaws(148) +ratelaws(149) +ratelaws(150) +ratelaws(191) +ratelaws(201) +ratelaws(202) +ratelaws(211) +ratelaws(212);
    Dspecies(2) = -ratelaws(4) -ratelaws(5) -ratelaws(6) -ratelaws(7) -ratelaws(8) -ratelaws(9) +ratelaws(22) +ratelaws(26) +ratelaws(27) +ratelaws(29) +ratelaws(31) +ratelaws(33) +ratelaws(200);
    Dspecies(3) = -ratelaws(1) -ratelaws(4) -ratelaws(5) +ratelaws(10) -ratelaws(19) -ratelaws(20) -ratelaws(21) +ratelaws(22) -ratelaws(23) -ratelaws(24) -ratelaws(25) +ratelaws(26) +ratelaws(44) -ratelaws(63) -ratelaws(64) -ratelaws(65) -ratelaws(66) -ratelaws(67) +ratelaws(68) +ratelaws(69) +ratelaws(70) +ratelaws(71) -ratelaws(72) -ratelaws(73) -ratelaws(74) -ratelaws(75) -ratelaws(76) +ratelaws(77) +ratelaws(78) +ratelaws(79) +ratelaws(80) -ratelaws(110) -ratelaws(153) -ratelaws(154) -ratelaws(155) -ratelaws(156) -ratelaws(157) +ratelaws(158) +ratelaws(159) +ratelaws(160) +ratelaws(161) +ratelaws(162) -ratelaws(163) -ratelaws(164) -ratelaws(165) -ratelaws(166) -ratelaws(167) +ratelaws(168) +ratelaws(169) +ratelaws(170) +ratelaws(171) +ratelaws(172) -ratelaws(219) -ratelaws(220) +ratelaws(221) +ratelaws(222) +ratelaws(223) +ratelaws(224) +ratelaws(225) +ratelaws(226) -ratelaws(227) -ratelaws(228) +ratelaws(229) +ratelaws(230) +ratelaws(231) +ratelaws(232) +ratelaws(233) +ratelaws(234);
    Dspecies(4) = -ratelaws(2) -ratelaws(3) -ratelaws(6) -ratelaws(7) -ratelaws(8) -ratelaws(9) +ratelaws(14) +ratelaws(18) +ratelaws(27) +ratelaws(29) +ratelaws(31) +ratelaws(33) +ratelaws(53) +ratelaws(62) +ratelaws(81) +ratelaws(84) +ratelaws(87) +ratelaws(90) -ratelaws(113) -ratelaws(134) -ratelaws(173) -ratelaws(175) -ratelaws(177) -ratelaws(179);
    Dspecies(5) = ratelaws(1) -ratelaws(10) -ratelaws(35);
    Dspecies(6) = ratelaws(2) -ratelaws(14) -ratelaws(15) -ratelaws(36) +ratelaws(59) -ratelaws(135) +ratelaws(145);
    Dspecies(7) = ratelaws(3) -ratelaws(11) -ratelaws(18) -ratelaws(37) +ratelaws(50) -ratelaws(114) +ratelaws(129);
    Dspecies(8) = ratelaws(4) -ratelaws(22) -ratelaws(23) -ratelaws(38) +ratelaws(77);
    Dspecies(9) = ratelaws(5) -ratelaws(19) -ratelaws(26) -ratelaws(41) +ratelaws(68);
    Dspecies(10) = ratelaws(6) -ratelaws(16) -ratelaws(24) -ratelaws(27) -ratelaws(28) -ratelaws(39) +ratelaws(60) +ratelaws(78) +ratelaws(83) -ratelaws(136) +ratelaws(151);
    Dspecies(11) = ratelaws(7) -ratelaws(12) -ratelaws(25) -ratelaws(29) -ratelaws(30) -ratelaws(40) +ratelaws(51) +ratelaws(79) +ratelaws(86) -ratelaws(115) +ratelaws(130);
    Dspecies(12) = ratelaws(8) -ratelaws(17) -ratelaws(20) -ratelaws(31) -ratelaws(32) -ratelaws(42) +ratelaws(61) +ratelaws(69) +ratelaws(89) -ratelaws(137) +ratelaws(152);
    Dspecies(13) = ratelaws(9) -ratelaws(13) -ratelaws(21) -ratelaws(33) -ratelaws(34) -ratelaws(43) +ratelaws(52) +ratelaws(70) +ratelaws(92) -ratelaws(116) +ratelaws(131);
    Dspecies(14) = ratelaws(11) +ratelaws(15) -ratelaws(50) -ratelaws(59) -ratelaws(93) -ratelaws(94);
    Dspecies(15) = ratelaws(12) -ratelaws(51) -ratelaws(72) -ratelaws(95) -ratelaws(99) +ratelaws(168);
    Dspecies(16) = ratelaws(13) -ratelaws(52) -ratelaws(63) -ratelaws(96) -ratelaws(108) +ratelaws(158);
    Dspecies(17) = ratelaws(16) -ratelaws(60) -ratelaws(73) -ratelaws(97) -ratelaws(100) +ratelaws(169);
    Dspecies(18) = ratelaws(17) -ratelaws(61) -ratelaws(64) -ratelaws(98) -ratelaws(109) +ratelaws(159);
    Dspecies(19) = ratelaws(19) +ratelaws(23) -ratelaws(68) -ratelaws(77) -ratelaws(101);
    Dspecies(20) = ratelaws(20) -ratelaws(54) -ratelaws(69) -ratelaws(102) -ratelaws(138) +ratelaws(146) +ratelaws(213);
    Dspecies(21) = ratelaws(21) -ratelaws(45) -ratelaws(70) -ratelaws(103) -ratelaws(117) +ratelaws(124) +ratelaws(203);
    Dspecies(22) = ratelaws(24) -ratelaws(55) -ratelaws(78) -ratelaws(104) -ratelaws(139) +ratelaws(147) +ratelaws(214);
    Dspecies(23) = ratelaws(25) -ratelaws(46) -ratelaws(79) -ratelaws(105) -ratelaws(118) +ratelaws(125) +ratelaws(204);
    Dspecies(24) = ratelaws(28) +ratelaws(34) -ratelaws(83) -ratelaws(92) -ratelaws(106);
    Dspecies(25) = ratelaws(30) +ratelaws(32) -ratelaws(86) -ratelaws(89) -ratelaws(107);
    Dspecies(26) = ratelaws(35) -ratelaws(44) +ratelaws(110);
    Dspecies(27) = ratelaws(36) -ratelaws(53) -ratelaws(56) +ratelaws(113) -ratelaws(140) +ratelaws(148) +ratelaws(210);
    Dspecies(28) = ratelaws(37) -ratelaws(47) -ratelaws(62) -ratelaws(119) +ratelaws(126) +ratelaws(134) +ratelaws(205);
    Dspecies(29) = ratelaws(38) -ratelaws(71) -ratelaws(74) +ratelaws(155) +ratelaws(170);
    Dspecies(30) = ratelaws(39) -ratelaws(57) -ratelaws(75) -ratelaws(81) -ratelaws(82) -ratelaws(141) +ratelaws(149) +ratelaws(171) +ratelaws(173) +ratelaws(174) +ratelaws(215);
    Dspecies(31) = ratelaws(40) -ratelaws(48) -ratelaws(76) -ratelaws(84) -ratelaws(85) -ratelaws(120) +ratelaws(127) +ratelaws(172) +ratelaws(175) +ratelaws(176) +ratelaws(206);
    Dspecies(32) = ratelaws(41) -ratelaws(65) -ratelaws(80) +ratelaws(160) +ratelaws(165);
    Dspecies(33) = ratelaws(42) -ratelaws(58) -ratelaws(66) -ratelaws(87) -ratelaws(88) -ratelaws(142) +ratelaws(150) +ratelaws(161) +ratelaws(177) +ratelaws(178) +ratelaws(216);
    Dspecies(34) = ratelaws(43) -ratelaws(49) -ratelaws(67) -ratelaws(90) -ratelaws(91) -ratelaws(121) +ratelaws(128) +ratelaws(162) +ratelaws(179) +ratelaws(180) +ratelaws(207);
    Dspecies(35) = ratelaws(44) +ratelaws(53) +ratelaws(62) -ratelaws(110) -ratelaws(113) -ratelaws(114) -ratelaws(115) -ratelaws(116) -ratelaws(117) -ratelaws(118) -ratelaws(119) -ratelaws(120) -ratelaws(121) -ratelaws(122) -ratelaws(123) +ratelaws(129) +ratelaws(130) +ratelaws(131) -ratelaws(134) -ratelaws(135) -ratelaws(136) -ratelaws(137) -ratelaws(138) -ratelaws(139) -ratelaws(140) -ratelaws(141) -ratelaws(142) -ratelaws(143) -ratelaws(144) +ratelaws(145) +ratelaws(151) +ratelaws(152) -ratelaws(191) +ratelaws(203) +ratelaws(204) +ratelaws(205) +ratelaws(206) +ratelaws(207) +ratelaws(208) +ratelaws(209) +ratelaws(210) +ratelaws(213) +ratelaws(214) +ratelaws(215) +ratelaws(216) +ratelaws(217) +ratelaws(218);
    Dspecies(36) = ratelaws(45) +ratelaws(63) -ratelaws(124) -ratelaws(158) -ratelaws(181) -ratelaws(192);
    Dspecies(37) = ratelaws(46) +ratelaws(72) -ratelaws(125) -ratelaws(168) -ratelaws(182) -ratelaws(193);
    Dspecies(38) = ratelaws(47) +ratelaws(94) -ratelaws(126) +ratelaws(135) -ratelaws(145) -ratelaws(183);
    Dspecies(39) = ratelaws(48) +ratelaws(99) -ratelaws(127) -ratelaws(163) -ratelaws(184) +ratelaws(229);
    Dspecies(40) = ratelaws(49) +ratelaws(108) -ratelaws(128) -ratelaws(153) -ratelaws(185) +ratelaws(221);
    Dspecies(41) = ratelaws(54) +ratelaws(64) -ratelaws(146) -ratelaws(159) -ratelaws(186) -ratelaws(194);
    Dspecies(42) = ratelaws(55) +ratelaws(73) -ratelaws(147) -ratelaws(169) -ratelaws(187) -ratelaws(195);
    Dspecies(43) = ratelaws(56) +ratelaws(93) +ratelaws(114) -ratelaws(129) -ratelaws(148) -ratelaws(188);
    Dspecies(44) = ratelaws(57) +ratelaws(100) -ratelaws(149) -ratelaws(164) -ratelaws(189) +ratelaws(232);
    Dspecies(45) = ratelaws(58) +ratelaws(109) -ratelaws(150) -ratelaws(154) -ratelaws(190) +ratelaws(224);
    Dspecies(46) = ratelaws(65) +ratelaws(74) +ratelaws(101) -ratelaws(160) -ratelaws(170);
    Dspecies(47) = ratelaws(66) +ratelaws(102) -ratelaws(132) -ratelaws(143) -ratelaws(161) +ratelaws(211) +ratelaws(217);
    Dspecies(48) = ratelaws(67) +ratelaws(103) -ratelaws(111) -ratelaws(122) -ratelaws(162) +ratelaws(201) +ratelaws(208);
    Dspecies(49) = ratelaws(71) +ratelaws(80) +ratelaws(81) +ratelaws(84) +ratelaws(87) +ratelaws(90) -ratelaws(155) -ratelaws(165) -ratelaws(173) -ratelaws(175) -ratelaws(177) -ratelaws(179) -ratelaws(200);
    Dspecies(50) = ratelaws(75) +ratelaws(104) -ratelaws(133) -ratelaws(144) -ratelaws(171) +ratelaws(212) +ratelaws(218);
    Dspecies(51) = ratelaws(76) +ratelaws(105) -ratelaws(112) -ratelaws(123) -ratelaws(172) +ratelaws(202) +ratelaws(209);
    Dspecies(52) = ratelaws(82) +ratelaws(91) +ratelaws(106) -ratelaws(174) -ratelaws(180);
    Dspecies(53) = ratelaws(85) +ratelaws(88) +ratelaws(107) -ratelaws(176) -ratelaws(178);
    Dspecies(54) = ratelaws(95) +ratelaws(115) -ratelaws(130) -ratelaws(166) -ratelaws(196) +ratelaws(230);
    Dspecies(55) = ratelaws(96) +ratelaws(116) -ratelaws(131) -ratelaws(156) -ratelaws(198) +ratelaws(222);
    Dspecies(56) = ratelaws(97) +ratelaws(136) -ratelaws(151) -ratelaws(167) -ratelaws(197) +ratelaws(233);
    Dspecies(57) = ratelaws(98) +ratelaws(137) -ratelaws(152) -ratelaws(157) -ratelaws(199) +ratelaws(225);
    Dspecies(58) = ratelaws(111) +ratelaws(153) +ratelaws(192) -ratelaws(201) -ratelaws(221) -ratelaws(235);
    Dspecies(59) = ratelaws(112) +ratelaws(163) +ratelaws(193) -ratelaws(202) -ratelaws(229) -ratelaws(236);
    Dspecies(60) = ratelaws(117) +ratelaws(156) +ratelaws(181) -ratelaws(203) -ratelaws(222) -ratelaws(239);
    Dspecies(61) = ratelaws(118) +ratelaws(166) +ratelaws(182) -ratelaws(204) -ratelaws(230) -ratelaws(240);
    Dspecies(62) = ratelaws(119) +ratelaws(140) +ratelaws(183) +ratelaws(188) -ratelaws(205) -ratelaws(210);
    Dspecies(63) = ratelaws(120) +ratelaws(184) +ratelaws(196) -ratelaws(206) -ratelaws(227) +ratelaws(231);
    Dspecies(64) = ratelaws(121) +ratelaws(185) +ratelaws(198) -ratelaws(207) -ratelaws(219) +ratelaws(223);
    Dspecies(65) = ratelaws(122) -ratelaws(208) +ratelaws(219) -ratelaws(223) +ratelaws(235) +ratelaws(239);
    Dspecies(66) = ratelaws(123) -ratelaws(209) +ratelaws(227) -ratelaws(231) +ratelaws(236) +ratelaws(240);
    Dspecies(67) = ratelaws(132) +ratelaws(154) +ratelaws(194) -ratelaws(211) -ratelaws(224) -ratelaws(237);
    Dspecies(68) = ratelaws(133) +ratelaws(164) +ratelaws(195) -ratelaws(212) -ratelaws(232) -ratelaws(238);
    Dspecies(69) = ratelaws(138) +ratelaws(157) +ratelaws(186) -ratelaws(213) -ratelaws(225) -ratelaws(241);
    Dspecies(70) = ratelaws(139) +ratelaws(167) +ratelaws(187) -ratelaws(214) -ratelaws(233) -ratelaws(242);
    Dspecies(71) = ratelaws(141) +ratelaws(189) +ratelaws(197) -ratelaws(215) -ratelaws(228) +ratelaws(234);
    Dspecies(72) = ratelaws(142) +ratelaws(190) +ratelaws(199) -ratelaws(216) -ratelaws(220) +ratelaws(226);
    Dspecies(73) = ratelaws(143) -ratelaws(217) +ratelaws(220) -ratelaws(226) +ratelaws(237) +ratelaws(241);
    Dspecies(74) = ratelaws(144) -ratelaws(218) +ratelaws(228) -ratelaws(234) +ratelaws(238) +ratelaws(242);

end


end
